<?php
namespace PME\Modules\Planejamento\Prato;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class ReplicacaoReceitaPorServicoController {
    protected $utilities;
    protected $connection;
    protected $receita;
    protected $prato;
    
    public function __construct($utilities, $connection, $receita, $prato) {
        $this->utilities  = $utilities;
        $this->connection = $connection;
        $this->receita = $receita;
        $this->prato   = $prato;
    }
    
    public function replicaReceitaPorServico(DTO\Request $request, DTO\Response $response) {
        $this->connection->beginTransaction();
        try {
            $row = (Array) $request->getRow();
            $origem = $row['ORIGEM'];
            $tpReceita = $origem['TPRECEITA'];
            $filialO = $origem['UNIDADE'];
            if ($filialO === 'X') $filialO = null;
            $servicoO = $origem['SERVICO'];
            if ($servicoO === 'X') $servicoO = null;
            $tpCardO = $origem['TIPOCARD_ACAO'];
            if ($tpCardO === 'X') $tpCardO  = null;
            $cdfilcentpme = $origem['CDFILCENTPME'];
            if ($cdfilcentpme === 'X') $cdfilcentpme  = null;
            $pratos = $row['PRATOS'];
            $servicos = $row['SERVICOS'];
            
            $nrorg = $this->utilities->getNrOrg();
            
            foreach ($servicos as $servico) {
                $percentual = isset($servico['PERCENTUAL']) ? $servico['PERCENTUAL'] : '';

                if (!$percentual || $percentual <= 0) {
                    throw new \Exception('valorPercentualNegativo');
                }
                
                $filialD = $servico['CDFILIAL'];
                $servicoD = $servico['CDSERVFILI'];
                
                foreach ($pratos as $prato) {
                    // EXCLUSÕES
                    // EXCLUI PRODUTOS DA RECEITA POR SERVIÇO SELECIONADO
                    // $produtosReceita = $this->retornaReceita(
                    //     'receserv', $prato['CDPRATO'], $prato['CDFILAUXPRAT'],
                    //     $filialD, $servicoD
                    // );
                    
                    $produtosReceita = $this->receita->returnRecipeServico($prato['CDPRATO'], $prato['CDFILAUXPRAT'], $filialD, $servicoD);
                    
                    if (!empty($produtosReceita)) {
                        foreach ($produtosReceita as $produtoReceita) {
                            $produtoReceita['RECIPEDEST'] = 'SERVICO';
                            $this->receita->deleteRecipe($produtoReceita);
                        }
                    }
                    
                    // EXCLUI DADOS DA RECEITA POR SERVIÇO SELECIONADO
                    $newReq = new DTO\Request\Row('',"","","");
                    $newReq->setParameter('plate', $prato['CDPRATO']);
                    $newReq->setParameter('filAux', $prato['CDFILAUXPRAT']);
                    $newReq->setParameter('cdfilial', $filialD);
  		            $newReq->setParameter('cdserv', $servicoD);
                    $this->prato->returnPratserv($newReq, $response);
                    $dados = $this->prato->getResponseLastDataSet($response)->getRows();
                    
                    if(!empty($dados)) {
                        $dados = $dados[0];
                        $dados['RECIPEDEST'] = 'SERVICO';
                        $this->receita->deletePlate($dados);
                    }
                    
                    // EXCLUI GRAMAGEM DO PRATO DA RECEITA POR SERVIÇO SELECIONADO
                    $this->prato->returnPratogramag($newReq, $response);
                    $gramagem = $this->prato->getResponseLastDataSet($response)->getRows();
                    if (!empty($gramagem)) {
                        $gramagem = $gramagem[0];
                        $gramagem['RECIPEDEST'] = 'GRAMAGEM';
                        $this->receita->deletePlate($gramagem);
                    }
                    
                    //INSERÇÕES
                    // TIRA DA RECEITA DE ORIGEM OS PRODUTOS QUE NÃO SÃO ATIVOS NA FILIAL
                    // DE DESTINO
                    $newRequest = new DTO\Request\Row('',"","","");
                    $newRequest->setParameter('plate', $prato['CDPRATO']);
                    $newRequest->setParameter('filAux', $prato['CDFILAUXPRAT']);
                    
                    switch ($tpReceita) {
                        case 'P':
                            $receitaO = $this->receita->returnRecipePadrao($prato['CDPRATO'], $prato['CDFILAUXPRAT']);
                            $this->prato->returnPratpadr($newRequest, $response);
                            $dadosReceitaO = $this->prato->getResponseLastDataSet($response)->getRows();
                            break;
                        case 'L':
                            $receitaO = $this->receita->returnRecipeLocal($prato['CDPRATO'], $prato['CDFILAUXPRAT'], $filialO);
                            $newRequest->setParameter('cdfilial', $filialO);
                            $this->prato->returnPratfili($newRequest, $response);
                            $dadosReceitaO = $this->prato->getResponseLastDataSet($response)->getRows();
                            break;
                        case 'S':
                            $receitaO = $this->receita->returnRecipeServico($prato['CDPRATO'], $prato['CDFILAUXPRAT'], $filialO, $servicoO);
                            $newRequest->setParameter('cdfilial', $filialO);
      		                $newRequest->setParameter('cdserv', $servicoO);
                            $this->prato->returnPratserv($newRequest, $response);
                            $dadosReceitaO = $this->prato->getResponseLastDataSet($response)->getRows();
                            break;
                        case 'T':
                            $receitaO = $this->receita->returnRecipeTpCard($prato['CDPRATO'], $prato['CDFILAUXPRAT'], $tpCardO, $cdfilcentpme);
                            $newRequest->setParameter('tipocard', $tpCardO);
                            $this->prato->returnPrattpcard($newRequest, $response);
                            $dadosReceitaO = $this->prato->getResponseLastDataSet($response)->getRows();
                            break;
                    }
                    
                    //Insere Receita
                    if(!empty($receitaO)) {
                        foreach ($receitaO as $prodReceitaO) {
                            $produtoInserir = array(
                                'NRORG' => $nrorg,
                                '__is_new' => true,
                                'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                                'CDPRATO' => $prato['CDPRATO'],
                                'CDFILIAL' => $filialD,
                                'CDSERVFILI' => $servicoD,
                                'CDPRODUTO' => $prodReceitaO['CDPRODUTO'],
                                'SGUNIDADE' => $prodReceitaO['SGUNIDADE']
                            );
                            
                            switch ($tpReceita) {
                                case 'P':
                                    $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODPRAT']   * $percentual / 100;
                                    $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCPR'];
                                    $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORPR'];
                                    $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTAPR']    * $percentual / 100;
                                    $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTPR']  * $percentual / 100;
                                    $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTPR'];
                                    $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRPR'] * $percentual / 100;
                                    $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTAPRUNR'] * $percentual / 100;
                                    $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPPR'];
                                    $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASEPR'];
                                    $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXPR'];
                                    $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRP'];
                                    $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODPADR'];
                                    $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODPADR'];
                                    $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                                    $produtoInserir['QTLIMSEGSERV']    = $prodReceitaO['QTLIMSEGPADR'];
                                    break;
                                case 'L':
                                    $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODRECLOC'] * $percentual / 100;
                                    $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCLO'];
                                    $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORLO'];
                                    $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTALO']    * $percentual / 100;
                                    $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTLO']  * $percentual / 100;
                                    $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTLO'];
                                    $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRLO'] * $percentual / 100;
                                    $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTALOUNR'] * $percentual / 100;
                                    $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPLO'];
                                    $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASELO'];
                                    $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXLO'];
                                    $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRF'];
                                    $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODFILI'];
                                    $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODFILI'];
                                    $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                                    $produtoInserir['QTLIMSEGSERV']    = $prodReceitaO['QTLIMSEGFILI'];
                                    break;
                                case 'S':
                                    $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODRECSER'] * $percentual / 100;
                                    $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCSE'];
                                    $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORSE'];
                                    $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTASE']    * $percentual / 100;
                                    $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTSE']  * $percentual / 100;
                                    $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTSE'];
                                    $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRSE'] * $percentual / 100;
                                    $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTASEUNR'] * $percentual / 100;
                                    $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPSE'];
                                    $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASESE'];
                                    $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXSE'];
                                    $produtoInserir['IDUTCOMPENTDIRS'] = $prodReceitaO['IDUTCOMPENTDIRS'];
                                    $produtoInserir['QTCOMPLPRODSERV'] = $prodReceitaO['QTCOMPLPRODSERV'];
                                    $produtoInserir['IDTPREQPRODSERV'] = $prodReceitaO['IDTPREQPRODSERV'];
                                    $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                                    $produtoInserir['QTLIMSEGSERV']    = $prodReceitaO['QTLIMSEGSERV'];
                                    break;
                                case 'T':
                                    $produtoInserir['QTPRODRECSER']    = $prodReceitaO['QTPRODPRATTP'] * $percentual / 100;
                                    $produtoInserir['VRPEFATCOCSE']    = $prodReceitaO['VRPEFATCOCPRTP'];
                                    $produtoInserir['VRPEFATCORSE']    = $prodReceitaO['VRPEFATCORPRTP'];
                                    $produtoInserir['QTBRUTASE']       = $prodReceitaO['QTBRUTAPRTP']    * $percentual / 100;
                                    $produtoInserir['QTUNIDESTSE']     = $prodReceitaO['QTUNIDESTPRTP']  * $percentual / 100;
                                    $produtoInserir['VRPEFATNUTSE']    = $prodReceitaO['VRPEFATNUTPRTP'];
                                    $produtoInserir['QTCALCNUTRSE']    = $prodReceitaO['QTCALCNUTRPRTP'] * $percentual / 100;
                                    $produtoInserir['QTBRUTASEUNR']    = $prodReceitaO['QTBRUTAPRUNRTP'] * $percentual / 100;
                                    $produtoInserir['HRTEMPPREPSE']    = $prodReceitaO['HRTEMPPREPPRTP'];
                                    $produtoInserir['IDPRODBASESE']    = $prodReceitaO['IDPRODBASEPRTP'];
                                    $produtoInserir['IDQTDEFIXSE']     = $prodReceitaO['IDQTDEFIXPRTP'];
                                    $produtoInserir['IDUTCOMPENTDIRS'] = 'N';
                                    $produtoInserir['QTCOMPLPRODSERV'] = null;
                                    $produtoInserir['IDTPREQPRODSERV'] = 'I';
                                    $produtoInserir['CDAPRERECE']      = $prodReceitaO['CDAPRERECE'];
                                    $produtoInserir['QTLIMSEGSERV']    = null;
                                    break;
                            }
                            
                            $produtoInserir['VRPERCDEGELO'] = $prodReceitaO['VRPERCDEGELO'];
                            
                            $this->receita->insertRecipeService($produtoInserir);
                        }
                    }
                    
                    // Insere dados da receita
                    if (!empty($dadosReceitaO)) {
                        $pratPadrInserir = array(
                            'NRORG'        => $nrorg,
                            '__is_new'     => true,
                            'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                            'CDPRATO'      => $prato['CDPRATO'],
                            'CDFILIAL'     => $filialD,
                            'CDSERVFILI'   => $servicoD,
                            'DSPRATO'      => ""
                        );
                        $dadosReceitaO = $dadosReceitaO[0];
                        
                        switch ($tpReceita) {
                            case 'P':
                                $pratPadrInserir['NRCORPRI']   = $dadosReceitaO['NRCORPRIPRAT'];
                                $pratPadrInserir['NRCORSEQ']   = $dadosReceitaO['NRCORSEQPRAT'];
                                $pratPadrInserir['NRCORAUX']   = $dadosReceitaO['NRCORAUXPRAT'];
                                $pratPadrInserir['QTPADRAO']   = $dadosReceitaO['QTPADRAOPRAT'] * $percentual / 100;
                                $pratPadrInserir['QTCONTAI']   = $dadosReceitaO['QTCONTAIPRAT'];
                                $pratPadrInserir['HRVALCON']   = $dadosReceitaO['HRVALCONPRAT'];
                                $pratPadrInserir['NMFANTASIA']   = $dadosReceitaO['NMFANTASIAPR'];
                                $pratPadrInserir['QTCARBOPR']   = $dadosReceitaO['QTCARBOPRPAD'];
                                $pratPadrInserir['QTLIPIDPR']   = $dadosReceitaO['QTLIPIDPRPAD'];
                                $pratPadrInserir['QTPROTEPR']   = $dadosReceitaO['QTPROTEPRPAD'];
                                $pratPadrInserir['QTCALORPR']   = $dadosReceitaO['QTCALORPRPAD'];
                                $pratPadrInserir['QTALCOOPR']   = $dadosReceitaO['QTALCOOPRPAD'];
                                $pratPadrInserir['VRPEMIN']    = $dadosReceitaO['VRPEMINPADR'];
                                $pratPadrInserir['QTAMOSTR']   = $dadosReceitaO['QTAMOSTRPADR'];
                                $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                                $pratPadrInserir['QTLIM']   = $dadosReceitaO['QTLIMSEGPADR'];
                                $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                                $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                                $pratPadrInserir['IDFINAL']    = $dadosReceitaO['IDFINALPADR'];
                                break;
                            case 'L':
                                $pratPadrInserir['NRCORPRI']   = $dadosReceitaO['NRCORPRIFILI'];
                                $pratPadrInserir['NRCORSEQ']   = $dadosReceitaO['NRCORSEQFILI'];
                                $pratPadrInserir['NRCORAUX']   = $dadosReceitaO['NRCORAUXFILI'];
                                $pratPadrInserir['QTPADRAO']   = $dadosReceitaO['QTPADRAOFILI'] * $percentual / 100;
                                $pratPadrInserir['QTCONTAI']   = $dadosReceitaO['QTCONTAIFILI'];
                                $pratPadrInserir['HRVALCON']   = $dadosReceitaO['HRVALCONFILI'];
                                $pratPadrInserir['NMFANTASIA']   = $dadosReceitaO['NMFANTASIAPF'];
                                $pratPadrInserir['QTCARBOPR']   = $dadosReceitaO['QTCARBOPRFIL'];
                                $pratPadrInserir['QTLIPIDPR']   = $dadosReceitaO['QTLIPIDPRFIL'];
                                $pratPadrInserir['QTPROTEPR']   = $dadosReceitaO['QTPROTEPRFIL'];
                                $pratPadrInserir['QTCALORPR']   = $dadosReceitaO['QTCALORPRFIL'];
                                $pratPadrInserir['QTALCOOPR']   = $dadosReceitaO['QTALCOOPRFIL'];
                                $pratPadrInserir['VRPEMIN']    = $dadosReceitaO['VRPEMINFILI'];
                                $pratPadrInserir['QTAMOSTR']   = $dadosReceitaO['QTAMOSTRFILI'];
                                $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                                $pratPadrInserir['QTLIM']   = $dadosReceitaO['QTLIMSEGFILI'];
                                $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                                $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                                $pratPadrInserir['IDFINAL']    = $dadosReceitaO['IDFINALFILI'];
                                break;
                            case 'S':
                                $pratPadrInserir['NRCORPRI']   = $dadosReceitaO['NRCORPRISERV'];
                                $pratPadrInserir['NRCORSEQ']   = $dadosReceitaO['NRCORSEQSERV'];
                                $pratPadrInserir['NRCORAUX']   = $dadosReceitaO['NRCORAUXSERV'];
                                $pratPadrInserir['QTPADRAO']   = $dadosReceitaO['QTPADRAOSERV'] * $percentual / 100;
                                $pratPadrInserir['QTCONTAI']   = $dadosReceitaO['QTCONTAISERV'];
                                $pratPadrInserir['HRVALCON']   = $dadosReceitaO['HRVALCONSERV'];
                                $pratPadrInserir['NMFANTASIA']   = $dadosReceitaO['NMFANTASIASE'];
                                $pratPadrInserir['QTCARBOPR']   = $dadosReceitaO['QTCARBOPRSER'];
                                $pratPadrInserir['QTLIPIDPR']   = $dadosReceitaO['QTLIPIDPRSER'];
                                $pratPadrInserir['QTPROTEPR']   = $dadosReceitaO['QTPROTEPRSER'];
                                $pratPadrInserir['QTCALORPR']   = $dadosReceitaO['QTCALORPRSER'];
                                $pratPadrInserir['QTALCOOPR']   = $dadosReceitaO['QTALCOOPRSER'];
                                $pratPadrInserir['VRPEMIN']    = $dadosReceitaO['VRPEMINSERV'];
                                $pratPadrInserir['QTAMOSTR']   = $dadosReceitaO['QTAMOSTRSERV'];
                                $pratPadrInserir['CDPRODPRODUCAO'] = $dadosReceitaO['CDPRODPRODUCAO'];
                                $pratPadrInserir['QTLIM']   = $dadosReceitaO['QTLIMSEGSERV'];
                                $pratPadrInserir['IDPRODCHILL']    = $dadosReceitaO['IDPRODCHILL'];
                                $pratPadrInserir['IDTPPRODCHILL']  = $dadosReceitaO['IDTPPRODCHILL'];
                                $pratPadrInserir['IDFINAL']    = $dadosReceitaO['IDFINALSERV'];
                                break;
                            case 'T':
                                $pratPadrInserir['NRCORPRI']   = $dadosReceitaO['NRCORPRIPRATTP'];
                                $pratPadrInserir['NRCORSEQ']   = $dadosReceitaO['NRCORSEQPRATTP'];
                                $pratPadrInserir['NRCORAUX']   = $dadosReceitaO['NRCORAUXPRATTP'];
                                $pratPadrInserir['QTPADRAO']   = $dadosReceitaO['QTPADRAOPRATTP'] * $percentual / 100;
                                $pratPadrInserir['QTCONTAI']   = $dadosReceitaO['QTCONTAIPRATTP'];
                                $pratPadrInserir['HRVALCON']   = $dadosReceitaO['HRVALCONPRATTP'];
                                $pratPadrInserir['NMFANTASIA']   = $dadosReceitaO['NMFANTASIAPRTP'];
                                $pratPadrInserir['QTCARBOPR']   = $dadosReceitaO['QTCARBOPRPADTP'];
                                $pratPadrInserir['QTLIPIDPR']   = $dadosReceitaO['QTLIPIDPRPADTP'];
                                $pratPadrInserir['QTPROTEPR']   = $dadosReceitaO['QTPROTEPRPADTP'];
                                $pratPadrInserir['QTCALORPR']   = $dadosReceitaO['QTCALORPRPADTP'];
                                $pratPadrInserir['QTALCOOPR']   = $dadosReceitaO['QTALCOOPRPADTP'];
                                $pratPadrInserir['VRPEMIN']    = $dadosReceitaO['VRPEMINPADRTP'];
                                $pratPadrInserir['QTAMOSTR']   = $dadosReceitaO['QTAMOSTRPADRTP'];
                                $pratPadrInserir['CDPRODPRODUCAO'] = null;
                                $pratPadrInserir['QTLIM']   = 0;
                                $pratPadrInserir['IDPRODCHILL']    = 'N';
                                $pratPadrInserir['IDTPPRODCHILL']  = 'NENHUM';
                                $pratPadrInserir['IDFINAL']    = 'M';
                                break;
                        }
                        $pratPadrInserir[0] = $pratPadrInserir;
                        $this->receita->insertPlateImport($pratPadrInserir, 'SERVICO');
                    }
                    
                    // Insere Gramagem por Prato
                    $gramagPratoInserir = array(
                        // 'NRORG' => $nrorg,
                        // '__is_new' => true,
                        'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                        'CDPRATO' => $prato['CDPRATO'],
                        'CDFILIAL' => $filialD,
                        'CDSERVFILI' => $servicoD,
                        'IDORIGEMRECE' => $tpReceita === 'L' ? 'F' : $tpReceita,
                        'CDFILIALORI' => $filialO,
                        'CDSERVFILIORI' => $servicoO,
                        'VRPERCENTUAL' => $percentual
                    );
                    $this->prato->insertPratoGramag($gramagPratoInserir);
                }
            }
            
            $this->connection->commit();
            $response->addDataset(new DataSet('RETORNO', array(true)));
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}